/******************************************************************************
 Multiple task user input example.
 This program illustrates how to create a task and use a mailbox to accept
 user input from a serial port. 

 1. UserMain task will wait pend on a mailbox for user input
 2. UserInput task will block waiting for a user to type in data from the serial port
 3. This example also illustrates how to terminate a task
 
 *****************************************************************************/
#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <ucos.h>
#include <serial.h>
#include <string.h>
#include <dhcpclient.h>
#include <netinterface.h>

//----- function prototypes -----
extern "C"
{
   void UserMain( void *pd );
}


//----- global vars -----
DWORD   UserInputTaskStack[USER_TASK_STK_SIZE];
OS_MBOX UserInputMbox;
BOOL    bExitUserInputTask;


/*-------------------------------------------------------------------
 UserInputTask
 Once created, this task will loop forever 
 ------------------------------------------------------------------*/
void UserInputTask( void *pdata )
{
   char strInput[100];

   while ( !bExitUserInputTask )
   {
      iprintf( "User Input:" );
      fgets(strInput, 80, stdin);
      
      // remove \r\n if present
      int len = strlen(strInput);
      iprintf("\r\n\r\nReceived %d bytes\r\n", len);
      if ( len >= 2 )
      {
         for (int i=0; i < 2; i++)
         {
            if ( ( strInput[len-i] == '\r' ) || ( strInput[len-i] == '\n' ) )
               strInput[len-i] = '\0';
         }
      }
      
      OSMboxPost( &UserInputMbox, ( void * ) strInput );
   }
   
   // Not much to do in this example once the task exits
   iprintf("*** UserInputTask() terminated. Please reset program ***\r\n");
}


/*-------------------------------------------------------------------
 UserMain
 ------------------------------------------------------------------*/
void UserMain( void *pd )
{
   InitializeStack();
   int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf( "\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
      iprintf("Static IP Address: ");
   ShowIP(ib->netIP); iprintf("\r\n");
   
   OSChangePrio( MAIN_PRIO );
   EnableAutoUpdate();

   iprintf( "Starting program\r\n" );

   OSMboxInit( &UserInputMbox, NULL );

   bExitUserInputTask = FALSE;
   if ( OSTaskCreate( UserInputTask,
                      NULL,
                      ( void * ) &UserInputTaskStack[USER_TASK_STK_SIZE],
                      ( void * ) UserInputTaskStack,
                      MAIN_PRIO + 1 ) != OS_NO_ERR )
   {
      iprintf( "*** Error creating task\r\n" );
   }

   while ( 1 )
   {
      BYTE err;

      // Wait forever for user input
      void *pmsg = OSMboxPend( &UserInputMbox, 0, &err );
      iprintf( "\r\nUserMain Received: \"%s\"\r\n\r\n", ( const char * ) pmsg );
      
      // If user input was "exit", then send exit signal to UserInputTask()
      if ( strcmp((const char *)pmsg, "exit") == 0 )
         bExitUserInputTask = TRUE;
   }
}











